@@ -9,7 +9,6 @@ from Scientific.IO import NetCDF
 from ncdfextract import nctypecode
 from ncdfproc import nccopydimension,nccopyattrvar
 
-
 class SomeError(Exception):
      def __init__(self, value):
          self.value = value
@@ -178,7 +177,7 @@ def readicecubeps(fstream,shp,refiter,dimiter,dimiterpos,refnoiter,dimnoiter,vty
 
 # we kunnen eens proberen om een variabele aan te maken met een vooraf gespecifieerde dimensie!
 
-def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes = 10000000):
+def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000):
     """ process binary data in order of specified dimensions
       func: the function to be used
     dnamsel: the dimensions on which the function needs to apply
@@ -194,7 +193,7 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
         # read in scipy.netcdf mode to obtain varariable offsets
         
         vsdin.append(dict())
-        if str(type(datin[idatin]['file']))[7:11] == 'list':
+        if type(datin[idatin]['file'])__name__ == 'list':
             ncfn =  datin[idatin]['file'][0]
             nctemp = netcdf_file(ncfn,'r')
             vsdin[idatin]['dnams'] = []
@@ -207,9 +206,12 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
                 vsdin[idatin]['dnams'].append(str(edim))
 
 	    print '1dnams', idatin,vsdin[idatin]['dnams']
-            # this statement could be a problem when defining daliases!!!
+            # this statement could be a problem when defining daliases for the file-list dimension?!!!
+            predim = None
+            if 'predim' in datin[idatin]:
+                predim = datin[idatin]['predim']
             if ((nctemp.variables[datin[idatin]['varname']].shape[0] == 1) & ((predim == None) | (nctemp.variables[datin[idatin]['varname']].dimensions[0] == predim))):
-                # we expand the first dimension
+                # we expand the first dimension (which only has length 1 per file)
                 vsdin[idatin]['dims'] = [len(datin[idatin]['file'])]+list(nctemp.variables[datin[idatin]['varname']].shape[1:])
 
             else: 
@@ -226,11 +228,11 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
 		
         else:
             # we assume a netcdf file
-            if str(type(datin[idatin]['file']))[7:17]  == 'NetCDFFile':
+            if type(datin[idatin]['file']).__name__  == 'NetCDFFile':
                 # obtain file name from open netcdf!! very nasty!!!
                 ncfn =  str(datin[idatin]['file'])[19:(str(datin[idatin]['file']).index("'",19))]
             # we assume a file name
-            elif str(type(datin[idatin]['file']))[7:10] == 'str':
+            elif type(datin[idatin]['file']).__name__ == 'str':
                 ncfn = datin[idatin]['file']
             else:
                 raise SomeError("Input file "+ str(datin[idatin]) + " ("+str(idatin)+")  could not be recognized.")
@@ -296,6 +298,7 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
                 # In dnamsstd, ednam should be just after the dimensions preceding ednams in dnams  
                 # # actually, we also want that, in dnamsstd, ednam should be just before the dimensions succeeding ednams in dnams. Sometimes, this is not possible at the same time. But it will be the case if that is possible when applying one of the criteria
                 idx = 0
+                # implement this also below, I mean that 
                 for idnam2,ednam2 in enumerate(dnamsstd):
                     if ednam2 in evsdin['dnams'][0:(idnam+1)]:
                         idx =  max(idx  ,dnamsstd.index(ednam2) + 1)
@@ -326,18 +329,21 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
                         else:
                             dimsstd[dnamsstd.index(ednam)] = max(dimsstd[dnamsstd.index(ednam)],vsdin[ivsdin]['dims'][idnam])
     
-    
+    # add the missing dimensions selected for the function. 
     idnam = 0
-    # add the missing dimensions selected for the function
+    
+    # TODO: the position of this dimension needs to be accordence to the
+    #       dimension-order suggested by the input(output?) netcdf files!  
+
     for idnamsel,ednamsel in enumerate(dnamsel):
         if ednamsel not in dnamsstd:
             dnamsstd.insert(idnam,ednamsel)
             dimsstd.insert(idnam,None) # to be defined from the function
-            idnam = idnam+1 # moet dit ook hier niet boven geimplementeerd worden?
+            idnam = idnam +1 
         else:
             idnam = dnamsstd.index(ednam)+1
     
-    print "dimsstd", dimsstd
+    print ("dimsstd", dimsstd)
     
     # dimsstd: list the specific output dimensions
     # if function dimension: data output dimension should be the same as the function output dimension, but this should be checked afterwards.
@@ -345,7 +351,6 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
     # # look what's the output dimension like. If the dimension is not in the output variable, we add a dummy 1-dimension
     # we need to create/list adimsstd also before!! And then append them with the missing dimensions, as dummy 1-dimensions. If that is not sufficient, we will just get an error message.
     
-    
     # get references to the standard output dimensions on which the function is applied
     refdfuncstd = []
     for idnamsel,ednamsel in enumerate(dnamsel):
@@ -369,7 +374,6 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
             else:
                 idnam = vsdin[ivsdin]['dnams'].index(ednamsstd) + 1
     
-    
     # do the same for the data output variables
     
     # # vsdin[ivsdin]['refdstd']: references of data stream dimensions (vsdin[..]['dnams'] to the standard dimensions (dnamsstd) 
@@ -383,8 +387,6 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
             vsdout[ivsdout]['dnams'] = dnamsstd
     
     # adimfuncin: the input dimensions of the function based on the refdfuncstd
-    
-    
     # adimfuncin: the dimensions of the function input
     # adimfuncin = np.zeros((len(vsdin),len(refdfuncstd)),dtype='int32') - 1
     
@@ -412,8 +414,6 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
         arefdfuncin.append([])
         for idnamsel,ednamsel in enumerate(dnamsel):
             arefdfuncin[ivsdin].append(vsdin[ivsdin]['dnams'].index(ednamsel))
-    
-        
 
     adimfuncin = []
     alendfuncin = []
@@ -472,12 +472,12 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
                     while ((not dimensionfound) & (idatin < (len(datin) ))):
                         templopen = False
                         # try to copy the dimension from the input data
-                        if str(type(datin[idatin]['file']))[7:17]  == 'NetCDFFile':
+                        if type(datin[idatin]['file']).__name__  == 'NetCDFFile':
                             nctemplate = datin[idatin]['file']
-                        elif str(type(datin[idatin]['file']))[7:10]  == 'str':
+                        elif type(datin[idatin]['file']).__name__  == 'str':
                             nctemplate = NetCDF.NetCDFFile(datin[idatin]['file'],'r')
                             templopen = True
-                        elif str(type(datin[idatin]['file']))[7:11]  == 'list':
+                        elif type(datin[idatin]['file']).__name__  == 'list':
                             nctemplate = NetCDF.NetCDFFile(datin[idatin]['file'][0],'r')
                             templopen = True
                         else:
@@ -499,7 +499,6 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
                                     nccopydimension(nctemplate,datout[iddout]['file'], ednams) 
                                     dimensionfound = True
 
-
                         if templopen:
                             nctemplate.close()
                         idatin = idatin + 1
@@ -526,22 +525,20 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
                     datout[iddout]['file'].createDimension(dnams[idim],vsdout[iddout]['dims'][idim])
             vsdout[iddout]['dnams'] = dnams
 
-
             datout[iddout]['file'].createVariable(datout[iddout]['varname'],vsdout[iddout]['dtype'][1],tuple(vsdout[iddout]['dnams']))
             # we should check this at the time the dimensions are not created
             if (vsdout[iddout]['dims'] != list(datout[iddout]['file'].variables[datout[iddout]['varname']].shape)):
                 raise SomeError("dimensions of output file ( "+str(vsdout[iddout]['dims'])+"; "+ str(vsdout[iddout]['dnams'])+") do not correspond with intended output dimension "+str(datout[iddout]['file'].variables[datout[iddout]['varname']].shape)+"; "+str(datout[iddout]['file'].variables[datout[iddout]['varname']].dimensions))
     
-    
     for idatin,edatin in enumerate(datin):
-        if str(type(datin[idatin]['file']))[7:17]  == 'NetCDFFile':
+        if type(datin[idatin]['file']).__name__  == 'NetCDFFile':
             # obtain file pointer!! very nasty!!
             ncfn =  str(datin[idatin]['file'])[19:(str(datin[idatin]['file']).index("'",19))]
             vsdin[idatin]['fp'] = open(ncfn,'r')
-        elif str(type(datin[idatin]['file']))[7:10]  == 'str':
+        elif type(datin[idatin]['file']).__name__  == 'str':
             ncfn = datin[idatin]['file']
             vsdin[idatin]['fp'] = open(ncfn,'r')
-        elif str(type(datin[idatin]['file']))[7:11]  == 'list':
+        elif type(datin[idatin]['file']).__name__  == 'list':
             # !!!!!if series/list of file names, then the file poniters will open when at read-time
             ncfn = datin[idatin]['file']
             vsdin[idatin]['fp'] = datin[idatin]['file']
@@ -561,9 +558,6 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
         nctemp.close()
     
     # # next: check whether the output variable dimensions (if already present) are not too large, otherwise raise error. + Construct final output dimension specs
-    
-
-
     adimfuncout = [[None]*len(refdfuncstd)]*(len(vsdout))
     alendfuncout = []
     for ivsdout,evsdout in enumerate(vsdout):
@@ -576,11 +570,8 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
             else:
                 adimfuncout[ivsdout][irefdfuncstd] = range(evsdout['dims'][vsdout[ivsdout]['refdstd'].index(erefdfuncstd)])
             alendfuncout[ivsdout] = alendfuncout[ivsdout]*len(adimfuncout[ivsdout][irefdfuncstd])
-
-
     
     # arefsin: references of the standard dimensions to the data stream dimensions
-    
     arefsin = []
     for ivsdin,evsdin in enumerate(vsdin):
         arefsin.append([None]*len(vsdin[ivsdin]['refdstd']))
@@ -590,7 +581,6 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
             arefsin[ivsdin][erefdstd] = irefdstd
     
     # arefsout: references of the standard dimensions to the data stream dimensions
-    
     arefsout = []
     for ivsdout,evsdout in enumerate(vsdout):
         arefsout.append([None]*len(vsdout[ivsdout]['refdstd']))
@@ -599,7 +589,7 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
         for irefdstd,erefdstd in enumerate(vsdout[ivsdout]['refdstd']):
             arefsout[ivsdout][erefdstd] = irefdstd
     
-    print "dnamsstd", dnamsstd
+    print ("dnamsstd", dnamsstd)
     if appenddim == True:
         membytes = 0
         # dsellen = len(dnamsel)
@@ -651,7 +641,6 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
                         arefdfuncout[ivsdout].insert(maxarefdfunc, arefsout[ivsdout][idnam])
                         adimfuncout[ivsdout].insert(maxarefdfunc,range(vsdout[ivsdout]['dims'][arefsout[ivsdout][idnam]]))
                         alendfuncout[ivsdout] = alendfuncout[ivsdout] *vsdout[ivsdout]['dims'][arefsout[ivsdout][idnam]]
-                    #dnamsel.insert(dsellen,dnamsstd[idnam])
     
                     # recalculate the amount of bytes
                     membytes = 0
@@ -684,8 +673,6 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
         adimnoiterout.append(list(adimfuncout[ivsdout]))
         arefdnoiterout.append(list(arefdfuncout[ivsdout]))
 
-    #dnamselnoiter = list(dnamsel)
-    
     # membytes: minimum total memory that will be used. We will the increase usage  when possible/allowed.
     membytes = 0
     for ivsdin,evsdin in enumerate(vsdin):
@@ -768,10 +755,10 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
     # # vsdin[ivsdin]['refdstd']: references of data stream dimensions (vsdin[..]['dnams'] to the standard dimensions (dnamsstd) 
     # dnamselnoiter: references
 
-    print 'dims in:',vsdin[ivsdin]['dims']
-    print 'dnams in:',vsdin[ivsdin]['dnams']
-    print 'dims out:',vsdout[ivsdin]['dims']
-    print 'dnams out:',vsdout[ivsdin]['dnams']
+    print ('dims in:',vsdin[ivsdin]['dims'])
+    print ('dnams in:',vsdin[ivsdin]['dnams'])
+    print ('dims out:',vsdout[ivsdin]['dims'])
+    print ('dnams out:',vsdout[ivsdin]['dnams'])
     
     # guess from residual dimensions that are not in refnoiterin
     refditerstd = []
@@ -829,11 +816,11 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
     rwchunksizein = [1]*len(vsdin)
     for ivsdin,evsdin in enumerate(vsdin):
         idim = len(vsdin[ivsdin]['dims'])-1
-        while ((idim in arefdnoiterin[ivsdin]) & (idim >= 0) & (vsdin[ivsdin]['dsel'][idim] == False) & ((str(type(datin[ivsdin]['file']))[7:11]  != 'list') | (idim != 0))):
+        while ((idim in arefdnoiterin[ivsdin]) & (idim >= 0) & (vsdin[ivsdin]['dsel'][idim] == False) & ((type(datin[ivsdin]['file']).__name__  != 'list') | (idim != 0))):
             # The inner dimensions just have to be referenced so not in correct order.  We know that they will be read in the correct order in the end
             rwchunksizein[ivsdin] = rwchunksizein[ivsdin]*vsdin[ivsdin]['dims'][idim]
             idim = idim - 1
-    print "rwchunksizeout", rwchunksizein
+    print ("rwchunksizeout", rwchunksizein)
     
 
     rwchunksizeout = [1]*len(vsdout)
@@ -843,7 +830,7 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
             # The inner dimensions just have to be referenced so not in correct order.  We know that they will be read in the correct order in the end
             rwchunksizeout[ivsdout] = rwchunksizeout[ivsdout]*vsdout[ivsdout]['dims'][idim]
             idim = idim - 1
-    print "rwchunksizein",rwchunksizeout
+    print ("rwchunksizein",rwchunksizeout)
     
     adimnoapplyout = []
     for ivsdout,evsdout in enumerate(vsdout):
@@ -997,11 +984,9 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, predim = None,maxmembytes =
         sys.stdout.write (str(j+1)+'/'+str(lenitermax))
     
     for ivsdin,evsdin in enumerate(vsdin):
-        if str(type(vsdin[ivsdin]['fp']))[7:11] == 'file':
+        if type(vsdin[ivsdin]['fp']).__name__ == 'file':
             vsdin[ivsdin]['fp'].close()
     for ivsdout,evsdout in enumerate(vsdout):
         vsdout[ivsdout]['fp'].close()
     print(' ')
 
-import pylab as pl
-
